-- John Deere 2040s by MiguelFontanar1
JD2040s = {};

function JD2040s.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Steerable, specializations);
end;

function JD2040s:load(savegame)
    
    self.setHydraulicDirection = SpecializationUtil.callSpecializationsFunction("setHydraulicDirection");
    
    local hydraulicsCount = Utils.getNoNil(getXMLInt(self.xmlFile, "vehicle.hydraulics#count"), 0);    
    self.hydraulics = {};    
    for i=1, hydraulicsCount do
        local hydraulicName = string.format("vehicle.hydraulics.hydraulic%d", i);        
        self.hydraulics[i] = {};        
        self.hydraulics[i].node = Utils.indexToObject(self.components, getXMLString(self.xmlFile, hydraulicName .. "#index"));
        self.hydraulics[i].punch = Utils.indexToObject(self.components, getXMLString(self.xmlFile, hydraulicName .. "#punch"));
        self.hydraulics[i].translationPunch = Utils.indexToObject(self.components, getXMLString(self.xmlFile, hydraulicName .. "#punchFixpoint"));
        self.hydraulics[i].fixPoint = Utils.indexToObject(self.components, getXMLString(self.xmlFile, hydraulicName .. "#fixpoint"));    
        local xUp, yUp, zUp = Utils.getVectorFromString(Utils.getNoNil(getXMLString(self.xmlFile, hydraulicName .. "#upVectors"),"0 1 0"));
        self.hydraulics[i].upVectors = {xUp, yUp, zUp};
        
        local ax, ay, az;        
        if self.hydraulics[i].punch ~= nil then
            ax, ay, az = getWorldTranslation(self.hydraulics[i].punch);
        else
--            ax, ay, az = getWorldTranslation(self.hydraulics[i].node);
        end;
        if self.hydraulics[i].translationPunch ~= nil then
            local bx, by, bz = getWorldTranslation(self.hydraulics[i].translationPunch);        
            self.hydraulics[i].punchDistance = Utils.vector3Length(ax-bx, ay-by, az-bz);
        end;
    end;
    
    self.setHydraulicTime = 30;

end;

function JD2040s:keyEvent(unicode, sym, modifier, isDown)
end;

function JD2040s:update(dt)

if self.isEntered then    
end; 

    if self:getIsActive() then            
        self.setHydraulicTime = 30;    
    end;

    if self.setHydraulicTime > 0 then
        for k,v in pairs(self.hydraulics) do 
            self:setHydraulicDirection(k);
        end;
        self.setHydraulicTime = self.setHydraulicTime - 1;
    end;    
     
end;        

function JD2040s:draw()
    g_currentMission:addExtraPrintText(g_i18n:getText("HIDRA_FEL"))
    g_currentMission:addExtraPrintText(g_i18n:getText("HIDRA_LE"))
end;

function JD2040s:onEnter()
end;

function JD2040s:onLeave()

end;
function JD2040s:delete()
end;

function JD2040s:mouseEvent(posX, posY, isDown, isUp, button)
end;

function JD2040s:setHydraulicDirection(index)

    local hydraulic = self.hydraulics[index];

    if hydraulic.fixPoint ~= nil then
        local ax, ay, az = getWorldTranslation(hydraulic.node);
        local bx, by, bz = getWorldTranslation(hydraulic.fixPoint);
        local x, y, z = worldDirectionToLocal(getParent(hydraulic.node), bx-ax, by-ay, bz-az);
        local xUp, yUp, zUp = unpack(hydraulic.upVectors);
        setDirection(hydraulic.node, x, y, z, xUp, yUp, zUp);
        if hydraulic.punch ~= nil then
            local distance = Utils.vector3Length(ax-bx, ay-by, az-bz);
            setTranslation(hydraulic.punch, 0, 0, distance-hydraulic.punchDistance);
        end;
    end;
end;

function JD2040s:validateAttacherJoint(implement, jointDesc, dt)
    return true;
end;
